
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>     /* for _delay_ms() */
#include "usbdrv.h"

static unsigned char usbData[8];

// œKp
#define hbyte(a) (*((uchar*)&(a)+1))
#define lbyte(a) (*((uchar*)&(a)))
typedef uchar ptr;
#define _ptr(p) ((uchar)(uint16_t)(p))
#define get(p) (*((uchar*)(uint16_t)(p)))
inline static uint8_t byte(uint8_t t) {return t;}


static void delay(uchar d) {
	while(d--);
	//_delay_loop_1(d);
}
static uint8_t wait=60; // 160

static uint8_t usi_trans(uint8_t data){
	USIDR=data;
	USISR=(1<<USIOIF);
	do{
		delay(wait);
		USICR=(1<<USIWM0)|(1<<USICS1)|(1<<USICLK)|(1<<USITC);
	} while(!(USISR&(1<<USIOIF)));

	return USIDR;
}

static inline void isp_command(uint8_t data){
	ptr b=_ptr(usbData);
	uchar i;
	for (i=0;i<4;i++) {
		get(b)=usi_trans(get(data));
		data++; b++;
	}
}

// USB Report descriptor
PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = {
	0x06, 0xA0, 0xFF,	// USAGE_PAGE (FFA0H Vendor defined)
	0x09, 0x01,			// USAGE (Vendor Usage 1)
	0xA1, 0x01,			// COLLECTION (Application)
		0x09, 0x02,			// USAGE (vendor defined)
		0xA1, 0x00,			// COLLECTION (Physical)
			0x06, 0xA1, 0xFF,	// USAGE_PAGE (vendor defined)
		// Input report
			0x09, 0x03,			// USAGE (vendor defined)
			0x09, 0x04,			// USAGE (vendor defined)
			0x15, 0x80,			// LOGICAL_MINIMUM (-128)
			0x25, 0x7F,			// LOGICAL_MAXIMUM (127)
			0x35, 0x00,			// PHYSICAL_MINIMUM (0)
			0x45, 0xFF,			// PHYSICAL_MAXIMUM (255)
			0x75, 0x08,			// REPORT_SIZE (8bit)
			0x95, 0x08,			// REPORT_COUNT (8 fields)
			0x81, 0x02,			// INPUT (Data, Variable, Absolute)
		// Output report
			0x09, 0x05,			// USAGE (vendor defined)
			0x09, 0x06,			// USAGE (vendor defined)
			0x15, 0x80,			// LOGICAL_MINIMUM (-128)
			0x25, 0x7F,			// LOGICAL_MAXIMUM (127)
			0x35, 0x00,			// PHYSICAL_MINIMUM (0)
			0x45, 0xFF,			// PHYSICAL_MAXIMUM (255)
			0x75, 0x08,			// REPORT_SIZE (8bit)
			0x95, 0x08,			// REPORT_COUNT (8 fields)
			0x91, 0x08,			// OUTPUT (Data, Variable, Absolute)
		0xC0,					// END_COLLECTION
	0xC0					// END_COLLECTION (Application)
};

uchar usbFunctionWrite(uchar *data, uchar len)
{
	static uchar page_mode;
	static uint16_t page_addr;
	uchar i;
	uint8_t cmd = data[0]&0xfe;
//	if ( data[0] == 0x01 ) { // TEST
		usbData[0] = data[1];
	if ( data[0] == 0x02 ) { // SET_STATUS
		PORTB = (PORTB&~0x1C) | (data[2]&0x10) | ((data[1]&7)<<2);
	} else if ( cmd==0x10) { // SPI
		isp_command(_ptr(data)+1);
	} else if ( data[0] == 20 ) { // Page set
		page_mode = data[1];
		page_addr = 0;
	} else if ( cmd==22 ) { // Page buf
		for(i=0;i<data[1];i++) {
			usi_trans(page_mode);
			usi_trans(hbyte(page_addr));
			usi_trans(lbyte(page_addr));
			usbData[i]=usi_trans(data[i+2]);
			if (page_mode&0x88) { // EEPROM or FlashH
				page_addr++;
				page_mode&=~0x08;
			} else {
				page_mode|=0x08;
			}
		}
	} else if ( data[0] == 60 ) { // Set wait
		wait=data[1];
	}

	if (data[0]&1) {
		usbSetInterrupt(usbData, 8);
	}

	return 8;
}


usbMsgLen_t usbFunctionSetup(uchar data[8])
{
    usbRequest_t    *rq = (void *)data;
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* HID class request */
	    if(rq->bRequest == USBRQ_HID_SET_REPORT)
          return USB_NO_MSG;  /* use usbFunctionWrite() to receive data from host */
    }
    return 0;
}


int main(void)
{
	DDRB = ~(1<<5);
	PORTB = (1<<4)|(1<<2);
	USICR=(1<<USIWM0)|(1<<USICS1)|(1<<USICLK);

	usbInit();
	sei();

	for(;;){    /* main event loop */
		usbPoll();
    }

}

